/////////////////////////////////////////////////////////////////////////////
// Name:        room.h
// tag:         header for a room of aBridge tables.
// Author:      David Roundy
// Modified by:
// Copyright:   (c) 2002 David Roundy
// Licence:   	GPL
//---------------------------------------------------------------------------
// Last modified: Jan 26 2002
/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/////////////////////////////////////////////////////////////////////////////
#ifndef _ROOM_H_
#define _ROOM_H_

// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

#ifndef WX_PRECOMP
#include <wx/wx.h>
#include <wx/socket.h>
#include <wx/string.h>
#endif

#include "tables.h"
#include "irc.h"

class ftpFileCache;

class aBridgeRoom: public wxFrame {
public:
  aBridgeRoom(wxFrame* frame, char* title, int x, int y, int w, int h,
              Irc *the_irc, wxString the_room = "#aBridge");
  virtual ~aBridgeRoom();
  
  void OnCloseWindow(wxCloseEvent& event);
  
  // Menu callbacks
  void MenuNewTable(wxCommandEvent& event);
  void Preferences(wxCommandEvent& event);
  void NewTable(wxString tablename, wxString topic);
  void JoinTable(wxString tablename);
  void Talk(wxCommandEvent& event);
  void SetTalkFocus(wxFocusEvent& evt);
  void ConnectToServer();
  void SayString(wxString str);
  void ProcessIRCmessage(IrcMsgEvent &evt);
  //  void ProcessTableEvent(GameEvent &evt);
  void HandleJoin(const wxString& where);
  void HandleSomeoneJoined(IrcMsgEvent &evt);
  void HandleIJoined(IrcMsgEvent &evt);

  void ProcessTablesEvent(TablesEvent &evt);

  DECLARE_EVENT_TABLE()
    
private:
  void SendTablesEvent(const wxString &type, const wxString &msg);

  ftpFileCache     *m_cache;
  wxString          m_player_pic_md5;

  wxMenuBar*		m_menuBar;
  aBridgeTables*    m_tables;
  wxTextCtrl*       m_log;
  wxTextCtrl*       m_talk;
  IrcChannel*       m_irc;

  // Room info
};

#endif
