/////////////////////////////////////////////////////////////////////////////
// Name:        table.h
// tag:         Header for an aBridge table. 
// Author:      David Roundy
// Modified by:
// Copyright:   (c) 2001,2002 David Roundy
// Licence:   	GPL
//---------------------------------------------------------------------------
// Last modified: Jan 26, 2002
/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/////////////////////////////////////////////////////////////////////////////

#ifndef _TABLE_H_
#define _TABLE_H_

// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

#ifndef WX_PRECOMP
#include <wx/wx.h>
#include <wx/socket.h>
#include <wx/string.h>
#endif

#include "cards.h"
#include "game.h"
#include "abridge.h"
#include "colors.h"
#include "bidreview.h"
#include "scorewin.h"
#include "irc.h"

class LastTrick;

class aBridgeTable: public wxFrame {
public:
  aBridgeTable(wxFrame* frame, const wxString &title, 
               int x, int y, int w, int h,
               Irc *the_irc, wxString the_room = "#aBridge");
  virtual ~aBridgeTable();
  
  void OnCloseWindow(wxCloseEvent& event);
  void OnMove(const wxMoveEvent& event);
  
  // Menu callbacks
  void ShowBids(wxCommandEvent& event);
  void ShowScores(wxCommandEvent& event);
  void ShowLastTrick(wxCommandEvent& event);
  void Claim(wxCommandEvent& event);
  void Talk(wxCommandEvent& event);
  void SetTalkFocus(wxFocusEvent& evt);
  void Sit(Seat seat);
  void SayString(wxString str);
  void ProcessIRCmessage(IrcMsgEvent &evt);
  void ProcessGameEvent(GameEvent &evt);

  void OnActivate(wxActivateEvent& event);
  
  DECLARE_EVENT_TABLE()
    
private:
  void DisableClaim();
  void EnableClaim();

  wxMenuBar*		m_menuBar;
  aBridgeGame*	    m_game;
  wxTextCtrl*       m_log;
  wxTextCtrl*       m_talk;
  IrcChannel*       m_irc;

  BidReview* m_bidreview;
  ScoreWin* m_scorewin;
  LastTrick* m_lasttrick;
};

#endif
